[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
# Coevolving networks for opinion and social dynamics in agent-based models
 	


This repository contains python code for the publication *Coevolving networks for opinion and social dynamics
in agent-based models*. DOI: https://doi.org/10.48550/arXiv.2407.00145 by Nataša Djurdjevac Conrad, 
Nhu Quang Vu and Sören Nagel. 

## Agent-based model

The python code in this repository can be used to simulate Agent-based models with a coevolutionary opinion and social dynamics. Features of the model are:
- visualization of the simulation 
- clustering of the agents 
- temporal network information 
- the option to add a potential to the social dynamics via Attractors or a potential landscape
- some support for data simulation

## GSS Data
In order to use the code with the data used for the comparison, download the data from https://gss.norc.org/get-the-data
and add them to the *data/GSS* directory.

1. Create an account for GSS data explorer
2. Go to Search GSS Variables and select the following variables: year, polview, partyid, helpsick, eqwlth ;  and use "Save to MyGSS"
3. Go to MyGSS and make sure that all the variables are *Added to Extract*. If not, select the variables and use the 
"Actions" field to add them to extraction. 
4. Select the "Extract"  tab.
5. Click on "Create Extract" and fill out the name and description of the extract under "1 Name Your Extract"
6. Under "2 Build Extract":  select the variables listed on the right side of the screen and then drag them (all at once) to "Choose Variables"
7. Under "3 Choose output options" select "Excel Workbook (data + metadata)"
8. Change Excel output options to *Values* instead of *Labels*
10. Click on the "Create Extract" button and you will be redirected to the "Extracts" Tab of MyGSS
Once the Extract has finished processing, select the Extract under "Completed" and click "Actions" and then "Download"
11. The Download contains a *result* folder with the *data*, *metadata* and *postprocessing* .json. Move the result folder
into the data dir of the Project and rename *result* to *GSS*

## Usage

The *demos* directory contains several demonstrations of the basic usage.

An example on how to create and simulate a model is shown in [demo_model_simulation.ipynb](demos/demo_model_simulation.ipynb). 
A Comparison of data and a data-based simulation of the model is shown in [demo_data_simulation.ipynb](demos/demo_data_simulation.ipynb). 

The most basic way to use the model is as follows:

```python
from src import Model

model = Model(N=100, R=0.15, alpha=20, beta=20, sigma_op=0.01, sigma_sp=0.01, delta_t=0.01,
              seed=123)
model.initialize_model()
model.simulate(100)
model.plot()
```

To generate the figures from the paper, execute the scripts e.g.
```bash
python scripts/paper_figures_single_runs.py
```

## Documentation
conda
The full documentation for this project can be found by opening the *docs/_build/html/index.html* file in your browser.
If there is anything unclear, please let me know, then I can update the documentation.

## SetUp and Dependencies
After *cloning* the repository, create the conda environment by running the installation scripts from within the project root.
On Windows run
```shell
.\setup\win_install_requirements.bat
```
or on Linux run
```shell
chmod +x setup/install_requirements.sh
./setup/install_requirements.sh
```
to install all the required packages and the necessary project structure.
After installing the dependencies make sure the conda environment is activated:
```shell
conda activate social_space
```

The scripts use LaTex, so it needs to be properly installed and added to the path.
## Project Structure
*src* contains the main modules.

There we have all the general functionality related to integrating and
plotting the model, as well as the *gss* submodule containing all the additional functionality need
to compare the model to the GSS dataset.

All the executable scripts used to output figures and data are located in *scripts*.

The *Images* folder contains all figures that are generated by the scripts.

All the tests are in *test*, but I am not sure, if we should even add them. They were reduced alot when i removed all 
the unneeded functionality. I could add some more tests form the new version.

The *docs* directory contains all the files needed to generate the documentation. In the tree below only the important
*index.html* file is highlighted. This file contains the documentation.

```
.
├── data
│  ├── DataBank
│  │  ├── GSS
│  │     ├── eqwlth_parameter_search.csv
│  │     └── help_sick_parameter_search.csv
│  └── GSS
│     ├── GSS.dat
│     ├── GSS.xlsx
│     └── post_processing_output.json
├── demos
│  ├── demo_data_simulation.ipynb
│  ├── demo_GSS.py
│  ├── demo_model_simulation.ipynb
│  ├── demo_simulation.py
│  └── demo_statistics.py
├── docs
│  ├── _build
│    ├── doctrees
│    └── html
│       └── index.html
├── Images
│  └── Paper
├── README.md
├── scripts
│  ├── hpc
│  │  └── submit.sh
│  ├── paper_figures_assortativity.py
│  ├── paper_figures_GSS_data.py
│  ├── paper_figures_potential.py
│  └── paper_figures_single_runs.py
├── setup
│  ├── environment.yml
│  ├── install_requirements.sh
│  ├── win_environment.yml
│  └── win_install_requirements.bat
├── src
│  ├── __init__.py
│  ├── agent.py
│  ├── bifurcations.py
│  ├── gss
│  │  ├── __init__.py
│  │  ├── Data_simulation_util.py
│  │  ├── GSS_analysis.py
│  │  └── image_util.py
│  ├── model.py
│  ├── model_plot_lib.py
│  ├── plotting_aux_file.py
│  └── statistics.py
└── test
   ├── test_model.py
   └── test_scripts.py

   
 ```
### An overview over the modules

- *agent.py* contains the Agent class that stores the data for of the individual agents
- *model.py* contains the main SDE integration and the analysis tools used for a single realization of the model
- *statistics.py* contains the Class that handles parralel integration of the model for a single
set of parameters and all the tools to analyse an ensemble of realizations.
- *bifurcations.py* handles parameter sweeps for one or two parameters


## Testing

To run all the scripts with reduced number of iterations and timesteps run
```shell
python -m unittest test_scripts.py
```
from within the *test* directory.


## Authors and acknowledgment
The authors of this project are Nataša Djurdjevac Conrad, 
Nhu Quang Vu and Sören Nagel.
The model was jointly developed by all three authors. The Python code has been written by Nhu Quang Vu and Sören Nagel.  

## Citation
If you use these files in your academic projects, we politely ask you to aknowledge it in your manuscript by the following BibTex citation:

*@misc{conrad2024coevolvingnetworksopinionsocial,
      title={Co-evolving networks for opinion and social dynamics in agent-based models}, 
      author={Nataša Djurdjevac Conrad and Nhu Quang Vu and Sören Nagel},
      year={2024},
      eprint={2407.00145},
      archivePrefix={arXiv},
      primaryClass={physics.soc-ph},
      url={https://arxiv.org/abs/2407.00145}, 
}*


