#!/bin/bash

# Get the directory in which the script is stored
script_dir="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"

# Change directory to the parent of the script's directory
cd "$script_dir/.." || exit

# Function to create and activate a conda environment from environment.yml
create_conda_environment() {
    if [ -f "setup/environment.yml" ]; then
        echo "Creating conda environment from environment.yml..."
        # Create the conda environment
        conda env create -f setup/environment.yml
        # Extract environment name from environment.yml
        env_name=$(head -n 1 setup/environment.yml | cut -d ' ' -f 2)
    else
        echo "environment.yml not found!"
        exit 1
    fi
}

# Function to create directories if they do not exist
create_directories() {
    local folders=("data/DataBank" "data/DataBank/GSS")
    for d in "${folders[@]}"; do
        if [ ! -d "$d" ]; then
            echo "Creating directory $d..."
            mkdir -p "$d"
        else
            echo "Directory $d already exists."
        fi
    done
}

# Function to create CSV files in the DataBank/GSS directory if they do not exist
create_csv_files() {
    local dir="data/DataBank/GSS"
    if [ -d "$dir" ]; then
        local files=("help_sick_parameter_search.csv" "eqwlth_parameter_search.csv")
        local headers="Goodness_of_Fit,R_op,R_sp,alpha,beta,sigma_op,sigma_sp"
        for file in "${files[@]}"; do
            local path="$dir/$file"
            if [ ! -f "$path" ]; then
                echo "Creating $file..."
                echo "$headers" > "$path"
            else
                echo "$file already exists."
            fi
        done
    else
        echo "data/DataBank/GSS directory does not exist!"
        exit 1
    fi
}

# Main function to execute the setup tasks
main() {
    create_conda_environment
    create_directories
    create_csv_files
    echo "Setup completed."
    read -p "Press Enter to continue..."
}

# Run the main function
main
