@echo off
setlocal enabledelayedexpansion

REM Get the directory in which the script is stored
set script_dir=%~dp0

REM Change directory to the parent of the script's directory
cd "%script_dir%.."

REM Function to create and activate a conda environment from win_environment.yml
:CREATE_CONDA_ENVIRONMENT
if exist "setup\win_environment.yml" (
    echo Creating conda environment from win_environment.yml...
    REM Create the conda environment
    conda env create -f setup\win_environment.yml
    REM Extract environment name from win_environment.yml
    for /f "tokens=2 delims= " %%a in ('findstr /r "^name:" setup\win_environment.yml') do set env_name=%%a
) else (
    echo win_environment.yml not found!
    exit /b 1
)

REM Function to create directories if they do not exist
:CREATE_DIRECTORIES
set "folders[0]=data\DataBank"
set "folders[1]=data\DataBank\GSS"

for %%i in (0,1) do (
    set "folder=!folders[%%i]!"
    if not exist "!folder!" (
        echo Creating directory !folder!...
        mkdir "!folder!"
    ) else (
        echo Directory !folder! already exists.
    )
)

REM Function to create CSV files in the DataBank\GSS directory if they do not exist
:CREATE_CSV_FILES
set "dir=data\DataBank\GSS"

if exist "!dir!" (
    set "files[0]=help_sick_parameter_search.csv"
    set "files[1]=eqwlth_parameter_search.csv"
    set "headers=Goodness_of_Fit,R_op,R_sp,alpha,beta,sigma_op,sigma_sp"

    for %%i in (0,1) do (
        set "file=!files[%%i]!"
        set "path=!dir!\!file!"
        if not exist "!path!" (
            echo Creating !file!...
            echo !headers! > "!path!"
        ) else (
            echo !file! already exists.
        )
    )
) else (
    echo data\DataBank\GSS directory does not exist!
    exit /b 1
)

REM Main function to execute the setup tasks
:MAIN
call :CREATE_CONDA_ENVIRONMENT
call :CREATE_DIRECTORIES
call :CREATE_CSV_FILES
echo Setup completed.
pause

REM Run the main function
call :MAIN

REM End script
exit /b 0
