import os
import sys
import glob
import unittest

# needed for the imports of our packages
sys.path.append(os.path.join(os.path.dirname(__file__), ".."))

import scripts.paper_figures_assortativity
import scripts.paper_figures_GSS_data
import scripts.paper_figures_single_runs
import scripts.paper_figures_potential


class TestScripts(unittest.TestCase):

    def setUp(self):
        self.test = True
        # Change the working directory to the script directory
        script_dir = os.path.dirname(os.path.abspath(__file__))
        os.chdir(script_dir)
        for i in range(1, 5):
            try:
                os.mkdir(f"Fig{i}")
            except FileExistsError:
                print(f"Fig{i} already exists.")

    def tearDown(self):
        # Change the working directory to the script directory
        script_dir = os.path.dirname(os.path.abspath(__file__))
        os.chdir(script_dir)
        # Delete all non-Python files in the test directory
        try:
            for file in glob.glob("*"):
                if not file.endswith(".py") and os.path.isfile(file):
                    os.remove(file)
            for i in range(1, 5):
                for file in glob.glob(f"Fig{i}/*"):
                    if not file.endswith(".py") and os.path.isfile(file):
                        os.remove(file)
                os.rmdir(f"Fig{i}")
        except FileNotFoundError:
            pass

    def test_assortativity_figures(self):
        # the first run will simulate everything
        scripts.paper_figures_assortativity.main(test=self.test)
        # the second run will load results from the first run
        scripts.paper_figures_assortativity.main(test=self.test)

    def test_gss_figures(self):
        scripts.paper_figures_GSS_data.main(test=self.test)

    def test_single_runs_figures(self):
        scripts.paper_figures_single_runs.main(test=self.test)

    def test_potential_figures(self):
        scripts.paper_figures_potential.main(test=self.test)
